from libc.stdint cimport uint8_t


cdef extern from '_hoedown/src/buffer.h':
    struct hoedown_buffer:
        uint8_t *data
        size_t size
        size_t asize
        size_t unit

    hoedown_buffer* hoedown_buffer_new(size_t)
    int hoedown_buffer_grow(hoedown_buffer *, size_t)
    void hoedown_buffer_cstr(hoedown_buffer *)
    void hoedown_buffer_free(hoedown_buffer *)
    void hoedown_buffer_puts(hoedown_buffer *, char *)


cdef extern from '_hoedown/src/html.h':
    hoedown_renderer *hoedown_html_renderer_new(
        unsigned int render_flags,
        int nesting_level)
    hoedown_renderer *hoedown_html_toc_renderer_new(
        int nesting_level)
    void hoedown_html_smartypants(
        hoedown_buffer *ob,
        uint8_t *text,
        size_t size)


cdef extern from '_hoedown/src/markdown.h':
    enum hoedown_autolink:
        pass

    struct hoedown_renderer:
        # Block level callbacks - NULL skips the block
        void (*blockcode)(hoedown_buffer *ob, hoedown_buffer *text, hoedown_buffer *lang, void *opaque)
        void (*blockquote)(hoedown_buffer *ob, hoedown_buffer *text, void *opaque)
        void (*blockhtml)(hoedown_buffer *ob, hoedown_buffer *text, void *opaque)
        void (*header)(hoedown_buffer *ob, hoedown_buffer *text, int level, void *opaque)
        void (*hrule)(hoedown_buffer *ob, void *opaque)
        void (*list)(hoedown_buffer *ob, hoedown_buffer *text, int flags, void *opaque)
        void (*listitem)(hoedown_buffer *ob, hoedown_buffer *text, int flags, void *opaque)
        void (*paragraph)(hoedown_buffer *ob, hoedown_buffer *text, void *opaque)
        void (*table)(hoedown_buffer *ob, hoedown_buffer *header, hoedown_buffer *body, void *opaque)
        void (*table_row)(hoedown_buffer *ob, hoedown_buffer *text, void *opaque)
        void (*table_cell)(hoedown_buffer *ob, hoedown_buffer *text, int flags, void *opaque)

        # Span level callbacks - NULL or return 0 prints the span verbatim
        int (*autolink)(hoedown_buffer *ob, hoedown_buffer *link, hoedown_autolink type, void *opaque)
        int (*codespan)(hoedown_buffer *ob, hoedown_buffer *text, void *opaque)
        int (*double_emphasis)(hoedown_buffer *ob, hoedown_buffer *text, void *opaque)
        int (*emphasis)(hoedown_buffer *ob, hoedown_buffer *text, void *opaque)
        int (*image)(hoedown_buffer *ob, hoedown_buffer *link, hoedown_buffer *title, hoedown_buffer *alt, void *opaque)
        int (*linebreak)(hoedown_buffer *ob, void *opaque)
        int (*link)(hoedown_buffer *ob, hoedown_buffer *link, hoedown_buffer *title, hoedown_buffer *content, void *opaque)
        int (*raw_html_tag)(hoedown_buffer *ob, hoedown_buffer *tag, void *opaque)
        int (*triple_emphasis)(hoedown_buffer *ob, hoedown_buffer *text, void *opaque)
        int (*strikethrough)(hoedown_buffer *ob, hoedown_buffer *text, void *opaque)
        int (*superscript)(hoedown_buffer *ob, hoedown_buffer *text, void *opaque)

        # Low level callbacks - NULL copies input directly into the output
        void (*entity)(hoedown_buffer *ob, hoedown_buffer *entity, void *opaque)
        void (*normal_text)(hoedown_buffer *ob, hoedown_buffer *text, void *opaque)

        # Header and footer
        void (*doc_header)(hoedown_buffer *ob, void *opaque)
        void (*doc_footer)(hoedown_buffer *ob, void *opaque)

        # state
        void *opaque

    enum hoedown_autolink:
        pass

    struct hoedown_markdown:
        pass

    hoedown_markdown *hoedown_markdown_new(
        unsigned int extensions,
        size_t max_nesting,
        hoedown_renderer *callbacks)
    void hoedown_markdown_render(
        hoedown_buffer *ob,
        uint8_t *document,
        size_t doc_size,
        hoedown_markdown *md)
    void hoedown_markdown_free(hoedown_markdown *md)
    void sd_version(int *major, int *minor, int *revision)



import os
import glob
import shutil
import os.path

try:
    from setuptools import setup, Extension, Command
except ImportError:
    from distutils.core import setup, Extension, Command


exec(open('hoedownpy/_version.py').read())
dirname = os.path.dirname(os.path.abspath(__file__))


class BaseCommand(Command):
    user_options = []
    def initialize_options(self):
        pass
    def finalize_options(self):
        pass


class CleanCommand(BaseCommand):
    description = 'cleanup directories created by packaging and build processes'
    def run(self):
        for path in ['build', 'dist', 'hoedownpy.egg-info', 'docs/_build', 'temp']:
            if os.path.exists(path):
                path = os.path.join(dirname, path)
                print('removing %s' % path)
                shutil.rmtree(path)


class CythonCommand(BaseCommand):
    description = 'compile Cython files(s) into C file(s)'
    def run(self):
        try:
            from Cython.Compiler.Main import compile
            for f in ("hoedown.pyx", "wrapper.pxd"):
                path = os.path.join(dirname, 'hoedownpy', f)
                print('compiling %s' % path)
                compile(path)
        except ImportError:
            print('Cython is not installed. Please install Cython first.')


class VendorCommand(BaseCommand):
    description = 'update hoedown files. Use `git submodule init`, '\
        '`git submodule update` and `git submodule foreach git pull origin master`'\
        ' to the most recent files'

    def run(self):
        os.system('git submodule update --init')
        os.system('git submodule foreach git pull origin master')
        files = []
        dest = os.path.join(dirname, 'hoedownpy/_hoedown/src')

        for path in ['vendor/hoedown/src/*', ]:
            files += glob.glob(os.path.join(dirname, path))

        for path in files:
            if os.path.exists(path):
                print('copy %s -> %s' % (path, dest))
                shutil.copy(path, dest)


class TestCommand(BaseCommand):
    description = 'run unit tests'
    def run(self):
        os.system('python tests/hoedown_test.py')


setup(
    name='hoedown',
    version=__version__,
    description='The Python binding for Hoedown, a markdown parsing library.',
    author='Hideo Hattori',
    author_email='hhatto.jp@gmail.com',
    url='https://github.com/hhatto/python-hoedown',
    license='MIT',
    long_description=open(os.path.join(dirname, 'README.rst')).read(),
    scripts=['scripts/hoedownpy'],
    cmdclass={
        'clean': CleanCommand,
        'compile_cython': CythonCommand,
        'update_vendor': VendorCommand,
        'test': TestCommand
    },
    ext_modules=[Extension('hoedown', [
        'hoedownpy/hoedown.c',
        'hoedownpy/wrapper.c',
        'hoedownpy/cb.c',
        'hoedownpy/_hoedown/src/html.c',
        'hoedownpy/_hoedown/src/stack.c',
        'hoedownpy/_hoedown/src/markdown.c',
        'hoedownpy/_hoedown/src/html_smartypants.c',
        'hoedownpy/_hoedown/src/html_blocks.c',
        'hoedownpy/_hoedown/src/escape.c',
        'hoedownpy/_hoedown/src/buffer.c',
        'hoedownpy/_hoedown/src/autolink.c'
    ], define_macros=[('inithoedownpy', 'inithoedown')])],
    classifiers = [
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: C',
        'Programming Language :: Cython',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.2',
        'Topic :: Text Processing :: Markup',
        'Topic :: Text Processing :: Markup :: HTML',
        'Topic :: Utilities'
    ]
)



from _hoedown cimport hoedown_buffer, hoedown_autolink, hoedown_renderer


cdef extern from *:
    ctypedef char* const_char_ptr "const char *"
    ctypedef char* const_size_t "const size_t"

cdef struct _toc_data:
    int header_count
    int current_level
    int level_offset
    int nesting_level

cdef struct rndr_state:
    _toc_data toc_data
    unsigned int flags
    void (*link_attributes)(hoedown_buffer *ob, const hoedown_buffer *url, void *self)


cdef extern from 'wrapper.h':

    struct renderopt:
        rndr_state html
        void *self

    hoedown_renderer callback_funcs
    const_char_ptr method_names[]
    const_size_t method_count
